/*******************************************************************************
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 *******************************************************************************/
package com.ofss.digx.appx.account;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.ofss.digx.app.account.dto.AccountRequestDTO;
import com.ofss.digx.app.account.dto.AccountResponseDTO;
import com.ofss.digx.app.context.ChannelContext;
import com.ofss.digx.app.core.ChannelInteraction;
import com.ofss.digx.appx.AbstractRESTApplication;
import com.ofss.digx.infra.exceptions.Exception;
import com.ofss.fc.infra.log.impl.MultiEntityLogger;

/**
 * Account REST class used to implement account related transactions API. 
 */
@Path("/account")
public class Account extends AbstractRESTApplication implements IAccount{

	/**
	 * Stores the name of the entity(class) represented by this {@code Class} object as a {@code String}
	 */
	private static final String THIS_COMPONENT_NAME = Account.class.getName();

	/**
	 * Instance of java.util.logging.Logger
	 */
	private static transient Logger LOGGER = MultiEntityLogger.getUniqueInstance().getLogger(THIS_COMPONENT_NAME);

	/**
	 * The formatter for formatting log messages.
	 */
	private static transient MultiEntityLogger formatter = MultiEntityLogger.getUniqueInstance();
	
	/**
	 * Calls the {@link com.ofss.digx.app.account.service.Account} class to fetch Account details from the system.
	 * The accountId is passed to find the respective details.
	 * 
	 * @param accountId
	 *            to uniquely identify the account details to be read
	 * @return An object of the type {@code javax.ws.rs.core.Response} containing status code 200 in case of successful REST call. 
	 */
	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/{accountId}")
	public Response fetch(@PathParam("accountId") String accountId, @QueryParam("bankCode") String bankCode, @QueryParam("isIban") boolean isIban, @QueryParam("isInternalAccount") boolean isInternalAccount) {
		if (LOGGER.isLoggable(Level.FINE)) {
			LOGGER.log(Level.FINE, formatter.formatMessage("Entering fetch in Account Class, Account Id: %s", accountId));
		}
		Response response = null;
		ChannelInteraction channelInteraction = null;
		ChannelContext channelContext = null;
		AccountRequestDTO dto = new AccountRequestDTO();		
		AccountResponseDTO responseDTO = new AccountResponseDTO();
		dto.setAccountId(accountId);
		dto.setBankCode(bankCode);
		dto.setIban(isIban);
		dto.setInternalAccount(isInternalAccount);			
		try {
			channelContext = super.getChannelContext();
			channelInteraction = ChannelInteraction.getInstance();
			channelInteraction.begin(channelContext);
		    com.ofss.digx.app.account.service.Account accountService = new com.ofss.digx.app.account.service.Account();
		    responseDTO = accountService.fetch(channelContext.getSessionContext(), dto);
			response = buildResponse(responseDTO, Response.Status.OK);
		} catch (Exception e) {
			LOGGER.log(Level.SEVERE,
					formatter.formatMessage(
							"Exception encountered while invoking the service %s while reading account details for  Account Id: %s",
							Account.class.getName(), accountId),
					e);
			response = buildResponse(e, Response.Status.BAD_REQUEST);
		} finally {
			try {
				channelInteraction.close(channelContext);
			} catch (Exception e) {
				LOGGER.log(Level.SEVERE, formatter.formatMessage("Exception while closing the channel interaction",
						Account.class.getName()), e);
				response = buildResponse(e, Response.Status.INTERNAL_SERVER_ERROR);
			}
		}
		if (LOGGER.isLoggable(Level.FINE)) {
			LOGGER.log(Level.FINE,
					formatter.formatMessage("Exiting fetch in %s, responseDTO: %s", THIS_COMPONENT_NAME, response));
		}
		return response;
	}
	
	
}
