/*******************************************************************************
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 *******************************************************************************/
package com.ofss.digx.app.account.service;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.ofss.digx.app.AbstractApplication;
import com.ofss.digx.app.Interaction;
import com.ofss.digx.app.account.assembler.AccountAssembler;
import com.ofss.digx.app.account.dto.AccountRequestDTO;
import com.ofss.digx.app.account.dto.AccountResponseDTO;
import com.ofss.digx.app.account.service.ext.IAccountExtExecutor;
import com.ofss.digx.app.ext.ServiceExtensionFactory;
import com.ofss.digx.domain.account.entity.policy.AccountBusinessPolicyDTO;
import com.ofss.digx.domain.account.entity.policy.AccountBusinessPolicyFactory;
import com.ofss.digx.infra.exceptions.Exception;
import com.ofss.fc.app.context.SessionContext;
import com.ofss.fc.framework.domain.policy.AbstractBusinessPolicy;
import com.ofss.fc.infra.log.impl.MultiEntityLogger;
import com.ofss.fc.service.response.TransactionStatus;


/**
 * Service implementation class to any account related transactions to be performed.
 */
public class Account extends AbstractApplication implements IAccount {

	/**
	 * Stores the name of the entity(class) represented by this {@code Class} object as a {@code String}
	 */
	private static final String THIS_COMPONENT_NAME = Account.class.getName();

	/**
	 * Instance of java.util.logging.Logger
	 */
	private static transient Logger LOGGER = MultiEntityLogger.getUniqueInstance().getLogger(THIS_COMPONENT_NAME);

	/**
	 * The formatter for formatting log messages.
	 */
	private static transient MultiEntityLogger FORMATTER = MultiEntityLogger.getUniqueInstance();
	/**
	 * Business policy constant
	 */
	private String ACCOUNT_BUSINESS_POLICY = "ACCOUNT_BUSINESS_POLICY";
	/**
	 * Factory implementation for the extension of this class. Any extension-method call on this instance, internally
	 * triggers a call to corresponding extension methods of all extension executor classes associated with the
	 * instance.
	 */
	private IAccountExtExecutor extensionExecutor = null;

	/**
	 * This is the Default Constructor for the class.
	 */
	public Account() {
		extensionExecutor = (IAccountExtExecutor) ServiceExtensionFactory
				.getServiceExtensionExecutor(Account.class.getName());
	}
	/**
	 * Fetches account related details after performing security, business policy and system constraint checks. Calls domain to fetch account related details.
	 * @param sessionContext login and other context details of the user.
	 * @param accountRequestDTO containing request details of user.
	 * @return AccountResponseDTO containing response details. 
	 * @throws Exception in case of any failure while fetching details.
	 */
@Override	
public AccountResponseDTO fetch(SessionContext sessionContext, AccountRequestDTO accountRequestDTO) throws Exception{
	if (LOGGER.isLoggable(Level.FINE)) {
		LOGGER.log(Level.FINE, FORMATTER.formatMessage("Entered into fetch() in class %s", THIS_COMPONENT_NAME));
	}
	
	AccountResponseDTO accountResponseDTO = new AccountResponseDTO();
	super.checkAccessPolicy("com.ofss.digx.app.account.service.Account.fetch", sessionContext,
			accountRequestDTO);
	Interaction.begin(sessionContext);
	TransactionStatus transactionStatus = fetchTransactionStatus();
	try {
		accountRequestDTO.validate(sessionContext);
		extensionExecutor.preFetch(sessionContext, accountRequestDTO);
		
		//assembler call for request DTO
		AccountAssembler assembler = new AccountAssembler();
		com.ofss.digx.domain.account.entity.Account accountDomain = new com.ofss.digx.domain.account.entity.Account();
		accountDomain = assembler.toDomainObject(accountRequestDTO);
	    
		//business policy check 
		AbstractBusinessPolicy abstractBusinessPolicy = null;
		AccountBusinessPolicyDTO accountBusinessPolicyDTO = new AccountBusinessPolicyDTO();
		accountBusinessPolicyDTO.setAccount(accountDomain);
		abstractBusinessPolicy = AccountBusinessPolicyFactory.getInstance().createPolicyInstance(
				ACCOUNT_BUSINESS_POLICY,
				accountBusinessPolicyDTO);
		abstractBusinessPolicy.validate();
		
		//assembler call for response DTO
		accountResponseDTO = assembler.fromDomainObject(accountDomain.fetch(accountDomain));
		extensionExecutor.postFetch(sessionContext, accountRequestDTO, accountResponseDTO);
		accountResponseDTO.setStatus(buildStatus(transactionStatus));
	} catch (Exception e) {
		fillTransactionStatus(transactionStatus, e);
		LOGGER.log(Level.SEVERE, FORMATTER.formatMessage(
				"Exception from fetch in class %s while generating response %s", THIS_COMPONENT_NAME,
				accountResponseDTO), e);
	} catch (RuntimeException rte) {
		fillTransactionStatus(transactionStatus, rte);
		LOGGER.log(Level.SEVERE, FORMATTER.formatMessage(
				"RuntimeException from fetch in class %s while generating response %s", THIS_COMPONENT_NAME,
				accountResponseDTO), rte);
	} finally {
		Interaction.close();
	}
	super.checkResponsePolicy(sessionContext, accountResponseDTO);
	if (LOGGER.isLoggable(Level.FINE)) {
		LOGGER.log(Level.FINE, FORMATTER.formatMessage(
				"Exiting from fetch() : accountResponseDTO= %s in class %s",
				accountResponseDTO, THIS_COMPONENT_NAME));
	}
	return accountResponseDTO;
}
}