/*******************************************************************************
 * Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
 *******************************************************************************/
/**
 * 
 */
package com.ofss.digx.app.account.service.ext;

import java.util.List;

import com.ofss.digx.app.account.dto.AccountRequestDTO;
import com.ofss.digx.app.account.dto.AccountResponseDTO;
import com.ofss.digx.app.account.service.Account;
import com.ofss.digx.app.ext.ServiceExtensionFactory;
import com.ofss.fc.app.context.SessionContext;


/**
 * Represents extension executor as hook point for {@link Account}.</br> Extensions can be called before/after executing
 * business logic of service. These methods are available in class with 'pre and post' as prefix of the actual method
 * name. The implementation of these methods depend upon the actual business requirement under different cases.
 */
public class AccountExtExecutor implements IAccountExtExecutor {

	/**
	 * Singleton object of class
	 */
	private static AccountExtExecutor accountExtExecutor = null;

	/**
	 * Stores extension hook point.
	 */
	private List<IAccountExt> extensions = null;

	/**
	 * default constructor
	 */
	@SuppressWarnings("unchecked")
	public AccountExtExecutor() {
		extensions = ServiceExtensionFactory.getServiceExtensions(Account.class.getName());
	}

	/**
	 * Returns singleton instance of type TermDepositExtExecutor
	 * 
	 * @return {@link TermDepositExtExecutor}
	 */
	public AccountExtExecutor getInstance() {
		if (accountExtExecutor == null) {
			accountExtExecutor = new AccountExtExecutor();
		}
		return accountExtExecutor;
	}
	
	/**
	 * Processes before the execution of actual business logic for fetching account details. Calls all the pre executional
	 * methods of all the extensions.
	 * 
	 * @param sessionContext
	 *            {@link SessionContext} containing session details
	 * @param accountRequest
	 *            {@link AccountRequestDTO} containing filters to fetch account list.
	 * @throws Exception
	 *             if any exception occurs.
	 */
	@Override
	public void preFetch(SessionContext sessionContext,
			AccountRequestDTO accountRequest) throws com.ofss.digx.infra.exceptions.Exception {
		for (int i = 0; i < extensions.size(); i++) {
			extensions.get(i).preFetch(sessionContext, accountRequest);
		}
		
	}

	/**
	 * Processes after the execution of actual business logic for fetching account details. Calls all the post executional
	 * methods of all the extensions.
	 * 
	 * @param sessionContext
	 *            {@link SessionContext} containing session details
	 * @param accountRequest
	 *            {@link AccountRequestDTO} containing filters to fetch account list.
	 * @param accountResponse
	 *            {@link AccountResponseDTO} containing account list.
	 * @throws Exception
	 *             if any exception occurs.
	 */
	@Override
	public void postFetch(SessionContext sessionContext,
			AccountRequestDTO accountRequest, AccountResponseDTO accountResponse)
			throws com.ofss.digx.infra.exceptions.Exception {
		for (int i = 0; i < extensions.size(); i++) {
			extensions.get(i).postFetch(sessionContext, accountRequest,accountResponse );
		}
	}

	

}
